﻿using AuditManager.Extensions;
using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Sync;
using AuditManager.Models.Tables;
using AuditManager.Shared.Constants;
using SQLite;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;

namespace AuditManager.Data
{
    public class SQLiteDatabase
    {
        readonly SQLiteAsyncConnection database;

        #region Constructors
        public SQLiteDatabase(string dbPath)
        {
            database = new SQLiteAsyncConnection(dbPath);
        }
        #endregion

        #region INIT
        public async Task<bool> CheckDatabaseStructureAsync()
        {
            var res = false;

            try
            {
                //recupera l'attuale struttura db locale
                List<SqliteMaster> tables = await database.QueryAsync<SqliteMaster>("SELECT * FROM sqlite_master WHERE type = 'table'", new object[] { });
                //recupera tutte le classi che si suppone verranno convertite in tabelle
                List<Type> tableModels = await ReflectionHelper.GetTypesBy(Constants.DatabaseAssembly, Constants.DatabaseNameSpace);

                foreach (var tm in tableModels)
                {
                    try
                    {
                        //verifica che abbia l'attributo Table                        
                        var tableName = tm.GetAttribute("TableAttribute")?.ConstructorArguments[0].Value.ToString();

                        if (!string.IsNullOrEmpty(tableName))
                        {
                            //potenzialmente è una tabella. Verifica che debba essere replicabile in locale
                            var replicate = tm.GetAttribute("TableReplicationAttribute")?.NamedArguments[0].TypedValue.Value;

                            if (replicate != null && (bool)replicate == true)
                            {
                                //ne verifico la presenza 
                                var table = tables.FirstOrDefault(w => w.name == tableName);
                                //se la tabella non è presente a db (locale) la crea
                                var create = table == null;

                                if (!create)
                                {
                                    //se presente, verifica la presenza di tutti i campi (contenuti nel campo sql della tabella sqlite_master)
                                    var properties = ReflectionHelper.GetPropertiesNames(tm);

                                    foreach (var p in properties)
                                    {
                                        //se la colonna non è presente, crea la tabella
                                        if (!table.sql.Contains(p))
                                        {
                                            create = true;
                                            break;
                                        }
                                    }
                                }

                                if (create)
                                {
                                    await database.CreateTablesAsync(CreateFlags.None, tm);
                                    res = true;
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        await FunctionsHelper.Log(nameof(CheckDatabaseStructureAsync), ex.Message);
                    }
                }

                await IndexDatabase();
            }
            catch (Exception ex)
            {
                await FunctionsHelper.Log(nameof(CheckDatabaseStructureAsync), ex.Message);
            }

            return res;
        }

        public async Task IndexDatabase()
        {
            string queryCreateIndex = "CREATE INDEX IF NOT EXISTS {0} ON {1} ({2})";

            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxDomande1", $"{nameof(AuditDomandeRisposte)}", $"{nameof(AuditDomandeRisposte.IdAudit)}, {nameof(AuditDomandeRisposte.IdCategoria)}"));
            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxDomande2", $"{nameof(AuditDomandeRisposte)}", $"{nameof(AuditDomandeRisposte.IdAudit)}, {nameof(AuditDomandeRisposte.IdCategoria)}, {nameof(AuditDomandeRisposte.IdTipoDomanda)}"));
            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxDomande3", $"{nameof(AuditDomandeRisposte)}", $"{nameof(AuditDomandeRisposte.IdAudit)}, {nameof(AuditDomandeRisposte.IdRisposta)}"));

            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxAllegatiRisposta1", $"{nameof(AuditAllegatiRisposta)}", $"{nameof(AuditAllegatiRisposta.IdAudit)}, {nameof(AuditAllegatiRisposta.IdRisposta)}, {nameof(AuditAllegatiRisposta.IdRispostaDettaglio)}, {nameof(AuditAllegatiRisposta.Attivo)}"));
            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxGrigliaAllegati1", $"{nameof(AuditGrigliaAllegati)}", $"{nameof(AuditGrigliaAllegati.IdAudit)}, {nameof(AuditGrigliaAllegati.IdRisposta)}, {nameof(AuditGrigliaAllegati.IdRispostaDettaglio)}, {nameof(AuditGrigliaAllegati.Attivo)}"));
            await database.ExecuteAsync(string.Format(queryCreateIndex, "IdxDocumentiEsaminati1", $"{nameof(AuditDocumentiEsaminati)}", $"{nameof(AuditDocumentiEsaminati.IdAudit)}, {nameof(AuditDocumentiEsaminati.IdRisposta)}, {nameof(AuditDocumentiEsaminati.IdRispostaDettaglio)}, {nameof(AuditDocumentiEsaminati.Attivo)}"));
        }

        public async Task<int> InsertTabelleAccessorie(TabelleAccessorieJson taj)
        {
            int retValue = 0;

            Dictionary<string, Type> classesOverTableName = new Dictionary<string, Type>();
            var tableModels = await ReflectionHelper.GetTypesBy(Constants.DatabaseAssembly, Constants.DatabaseNameSpace);

            foreach (var tm in tableModels)
            {
                var tableName = tm.GetAttribute("TableAttribute");

                if (tableName != null)
                    classesOverTableName.Add(tableName.ConstructorArguments[0].Value.ToString(), Type.GetType(tm.AssemblyQualifiedName));
            }

            if (taj.Sync != null)
            {
                foreach (var p in taj.Sync.GetType().GetRuntimeProperties())
                {
                    if ((bool.Parse(taj.Sync.GetType().GetRuntimeProperty(p.Name).GetValue(taj.Sync)?.ToString())) == true)
                    {
                        try
                        {
                            TableMapping mapping = await database.GetMappingAsync(classesOverTableName[p.Name]);

                            if (p.Name.Equals(nameof(ParametriGlobali)))
                                await database.ExecuteAsync($"DELETE FROM {nameof(ParametriGlobali)} WHERE DescParametro <> '{Constants.ConstPortaAngolometro}'");
                            else
                                await database.DeleteAllAsync(mapping);

                            await FunctionsHelper.Log(nameof(InsertTabelleAccessorie), $"{classesOverTableName[p.Name].Name.ToString()} tabella {p.Name} campi");

                            var v = (IEnumerable)taj.GetType().GetRuntimeProperty(p.Name).GetValue(taj);
                            await database.InsertAllAsync(v);

                            retValue++;
                        }
                        catch (Exception ex)
                        {
                            await FunctionsHelper.Log(nameof(InsertTabelleAccessorie), ex.Message);
                            retValue = -1;
                            break;
                        }
                    }
                }
            }

            return retValue;
        }

        public async Task<int> InsertCatenaAudit(TabellaJson djl, bool isLogin = false)
        {
            int retValue = 0;

            if (djl.Audit.Count > 0)
            {
                bool doDownload = true;

                try
                {
                    if (!isLogin)
                    {
                        foreach (var checkItem in djl.Audit)
                        {
                            int sommaConteggioNonSincronizzati = await ContaElementiNonSincronizzatiAsync(checkItem.IdAudit);

                            if (sommaConteggioNonSincronizzati > 0)
                            {
                                doDownload = false;
                                break;
                            }
                        }
                    }

                    if (doDownload)
                    {
                        foreach (var item in djl.Audit)
                        {
                            int contoAudit = await database.Table<Audit>().Where(w => w.IdAudit == item.IdAudit).CountAsync();

                            if (contoAudit > 0)
                            {
                                await UpdateAudit(item);
                                await DeleteCascadeAsync(item.IdAudit);
                            }
                            else
                                await database.InsertAsync(item);

                            if (item.IdStato == 4)
                            {
                                //int contoCompletato = await database.Table<AuditCompletato>().Where(x => x.IdAudit == item.IdAudit).CountAsync();

                                //if (contoCompletato == 0)
                                //{
                                //    AuditCompletato ac = new AuditCompletato();
                                //    ac.IdAudit = item.IdAudit;

                                //    database.Insert(ac);
                                //}
                            }
                        }

                        await database.InsertAllAsync(djl.AuditFirme);
                        await database.InsertAllAsync(djl.AuditTipiAudit);
                        await database.InsertAllAsync(djl.AuditUtenti);
                        await database.InsertAllAsync(djl.AuditMotivi);
                        await database.InsertAllAsync(djl.AuditChecklist);
                        await database.InsertAllAsync(djl.AuditCategorie);
                        await database.InsertAllAsync(djl.AuditCheckCategorie);
                        await database.InsertAllAsync(djl.AuditAllegati);
                        await database.InsertAllAsync(djl.AuditPersone);
                        await database.InsertAllAsync(djl.AuditServizi);
                        await database.InsertAllAsync(djl.AuditSoggetti);
                        await database.InsertAllAsync(djl.AuditDomandeRisposte);
                        await database.InsertAllAsync(djl.AuditRisposteDettaglio);
                        await database.InsertAllAsync(djl.AuditAuditor);
                        await database.InsertAllAsync(djl.AuditAllegatiRisposta);
                        await database.InsertAllAsync(djl.AuditGrigliaAllegati);
                        await database.InsertAllAsync(djl.AuditDocumentiEsaminati);
                        await database.InsertAllAsync(djl.AuditRisposteFirme);
                        await database.InsertAllAsync(djl.AuditDomandeRelated);

                        retValue = djl.Audit.Count;
                    }
                }
                catch (Exception ex)
                {
                    await FunctionsHelper.Log(nameof(InsertCatenaAudit), ex.Message);
                    retValue = -1;
                }
            }

            return retValue;
        }
        #endregion

        #region Log
        public async Task<bool> InsertLogAsync(Log log)
        {
            try
            {
                await database.InsertAsync(log);
                return true;
            }
            catch (Exception ex)
            {
                await FunctionsHelper.Log(nameof(InsertParametroAsync), ex.Message, false);
                return false;
            }
        }
        #endregion

        #region ParametriGlobali
        public async Task<bool> InsertParametroAsync(ParametriGlobali parametro)
        {
            try
            {
                await database.InsertAsync(parametro);
                return true;
            }
            catch (Exception ex)
            {
                await FunctionsHelper.Log(nameof(InsertParametroAsync), ex.Message);
                return false;
            }
        }

        public async Task<ParametriGlobali> GetParametroGlobale(string parametro)
        {
            if (parametro == null)
                return null;

            return await database.Table<ParametriGlobali>().Where(w => w.DescParametro == parametro).FirstOrDefaultAsync();
        }

        public async Task UpdateParametroAsync<T>(string parametro, object value)
        {
            ParametriGlobali pg = await database.Table<ParametriGlobali>().Where(x => x.DescParametro.Equals(parametro)).FirstOrDefaultAsync();

            if (value != null)
            {
                if (pg != null)
                {
                    if (typeof(T) == (typeof(int)))
                        pg.ValoreParametroInt = (int)value;
                    if (typeof(T) == (typeof(decimal)))
                        pg.ValoreParametroFloat = (double)value;
                    if (typeof(T) == (typeof(string)))
                        pg.ValoreParametroString = value.ToString();
                    if (typeof(T) == typeof(DateTime))
                        pg.ValoreParametroDate = (DateTime)value;

                    await database.UpdateAsync(pg);
                }
                else
                {
                    pg = new ParametriGlobali { DescParametro = parametro };

                    if (typeof(T) == (typeof(int)))
                        pg.ValoreParametroInt = (int)value;
                    if (typeof(T) == (typeof(decimal)))
                        pg.ValoreParametroFloat = (double)value;
                    if (typeof(T) == (typeof(string)))
                        pg.ValoreParametroString = value.ToString();
                    if (typeof(T) == typeof(DateTime))
                        pg.ValoreParametroDate = (DateTime)value;

                    await InsertParametroAsync(pg);
                }
            }
        }
        #endregion

        #region Audit
        public async Task<Audit> GetAudit(int idAudit)
        {
            return await database.Table<Audit>().Where(w => w.IdAudit == idAudit).FirstOrDefaultAsync();
        }

        public async Task<List<Audit>> GetAudits()
        {
            return await database.Table<Audit>().Where(w => w.Attivo == true).OrderBy(o => o.DataInizioAudit).ThenBy(t => t.DescAudit).ToListAsync();
        }

        public async Task<List<Audit>> GetAuditFilteredAsync(List<TipiAudit> listaTipiAuditIn, List<Soggetti> listaSoggetti, List<Motivazioni> listaMotivazioniIn)
        {
            List<Audit> listaAudit = await GetAudits();

            if (listaTipiAuditIn.Count > 0)
            {
                int[] listaTipiAuditArr = listaTipiAuditIn.Select(s => s.IdTipoAudit).ToArray();
                List<AuditTipiAudit> listaTipiAudit = await GetAuditTipiAudit(listaTipiAuditArr);
                listaAudit = listaAudit.Where(w => listaTipiAudit.Select(s => s.IdAudit).Contains(w.IdAudit)).ToList();
            }

            if (listaSoggetti.Count > 0)
            {
                string[] listaSoggettiArr = listaSoggetti.Select(s => s.Filtro).ToArray();
                listaAudit = listaAudit.Where(w => listaSoggettiArr.Contains(w.FiltroSoggetto)).ToList();
            }

            if (listaMotivazioniIn.Count > 0)
            {
                int[] listaMotivazioniArr = listaMotivazioniIn.Select(s => s.IdMotivazione).ToArray();
                List<AuditMotivi> listaMotivazioni = await GetAuditMotivi(listaMotivazioniArr);
                listaAudit = listaAudit.Where(w => listaMotivazioni.Select(s => s.IdAudit).Contains(w.IdAudit)).ToList();
            }

            return listaAudit;
        }

        public async Task<List<AuditTipiAudit>> GetAuditTipiAudit(int[] idTipoAuditArr)
        {
            return await database.Table<AuditTipiAudit>().Where(w => idTipoAuditArr.Contains(w.IdTipoAudit)).ToListAsync();
        }

        public async Task<List<AuditTipiAudit>> GetAuditTipiAudit(int idAudit)
        {
            return await database.Table<AuditTipiAudit>().Where(w => w.IdAudit == idAudit).ToListAsync();
        }

        public async Task<List<AuditMotivi>> GetAuditMotivi(int[] idMotivazioneArr)
        {
            return await database.Table<AuditMotivi>().Where(w => idMotivazioneArr.Contains(w.IdMotivazione)).ToListAsync();
        }

        public async Task<AuditCategorie> GetAuditCategoria(int pkid)
        {
            return await database.FindAsync<AuditCategorie>(pkid);
        }

        public async Task<AuditCategorie> GetAuditCategoria(int idAudit, int idCategoria)
        {
            return await database.Table<AuditCategorie>().Where(w => w.IdAudit == idAudit && w.IdCategoria == idCategoria).FirstOrDefaultAsync();
        }

        public async Task<List<AuditCategorie>> GetAuditCategorie(int idAudit)
        {
            return await database.Table<AuditCategorie>().Where(w => w.IdAudit == idAudit).OrderBy(o => o.DescCategoria).ToListAsync();
        }

        public async Task<AuditChecklist> GetAuditChecklist(int pkid)
        {
            return await database.FindAsync<AuditChecklist>(pkid);
        }

        public async Task<AuditChecklist> GetAuditChecklist(int idAudit, int idCheckList)
        {
            return await database.Table<AuditChecklist>().Where(w => w.IdAudit == idAudit && w.IdCheckList == idCheckList).FirstOrDefaultAsync();
        }

        public async Task<int> GetCountersAudit(int idAudit, string risposta)
        {
            int retValue = 0;
            string query = string.Empty;
            string listaTipiDomanda = "('1', '2', '9', '10', '11')";

            try
            {
                if (!string.IsNullOrEmpty(risposta))
                {
                    List<AuditDomandeRisposte> listaDomandeConRisposta = await database.Table<AuditDomandeRisposte>()
                        .Where(w => w.IdAudit == idAudit && w.IdDomandaParent == null && w.Attivo == true && 
                              (w.Risposta == risposta || ((w.Risposta == "" || w.Risposta == null) && w.RisposteDft == risposta)))
                        .ToListAsync();

                    retValue = listaDomandeConRisposta.Count();
                }
                else
                {
                    List<AuditDomandeRisposte> listaDomande = await database.Table<AuditDomandeRisposte>()
                        .Where(w => w.IdAudit == idAudit && w.IdDomandaParent == null && w.Attivo == true)
                        .ToListAsync();

                    //int totaleDomande = listaDomande.Count();
                    int domandeNonRisposte = listaDomande.Where(w => (w.Risposta == risposta || w.Risposta == null) && (w.RisposteDft == risposta || w.RisposteDft == null)).Count();

                    query = $"SELECT COUNT(*) AS totaleAllegati FROM {nameof(AuditAllegatiRisposta)} g, {nameof(AuditDomandeRisposte)} h " +
                        $"WHERE g.{nameof(AuditAllegatiRisposta.IdAudit)} = h.{nameof(AuditDomandeRisposte.IdAudit)} " +
                        $"AND g.{nameof(AuditAllegatiRisposta.IdRisposta)} = h.{nameof(AuditDomandeRisposte.IdRisposta)} " +
                        $"AND (h.{nameof(AuditDomandeRisposte.Risposta)} = '{risposta}' OR h.{nameof(AuditDomandeRisposte.Risposta)} IS NULL) AND " +
                        $"h.{nameof(AuditDomandeRisposte.IdTipoDomanda)} NOT IN {listaTipiDomanda} AND h.{nameof(AuditDomandeRisposte.IdDomandaParent)} IS NULL " +
                        $"AND h.{nameof(AuditDomandeRisposte.IdAudit)} = {idAudit} AND h.{nameof(AuditDomandeRisposte.Attivo)} = 1 AND g.{nameof(AuditDomandeRisposte.Attivo)} = 1";

                    int totaleAllegati = (await database.QueryAsync<int>(query)).FirstOrDefault();

                    retValue = domandeNonRisposte + totaleAllegati;
                }
            }
            catch (Exception ex)
            {
                retValue = 0;
            }

            return retValue;
        }

        public async Task<int> GetCountersAudit(int idAudit, int idCategoria, string risposta)
        {
            int retValue = 0;
            string query = string.Empty;
            string listaTipiDomanda = "('1', '2', '9', '10', '11')";

            try
            {
                if (!string.IsNullOrEmpty(risposta))
                {
                    List<AuditDomandeRisposte> listaDomandeConRisposta = await database.Table<AuditDomandeRisposte>()
                        .Where(w => w.IdAudit == idAudit && w.IdDomandaParent == null && w.Attivo == true && w.IdCategoria == idCategoria &&
                              (w.Risposta == risposta || ((w.Risposta == "" || w.Risposta == null) && w.RisposteDft == risposta)))
                        .ToListAsync();

                    retValue = listaDomandeConRisposta.Count();
                }
                else
                {
                    List<AuditDomandeRisposte> listaDomande = await database.Table<AuditDomandeRisposte>()
                        .Where(w => w.IdAudit == idAudit && w.IdDomandaParent == null && w.Attivo == true && w.IdCategoria == idCategoria)
                        .ToListAsync();

                    //int totaleDomande = listaDomande.Count();
                    int domandeNonRisposte = listaDomande.Where(w => (w.Risposta == risposta || w.Risposta == null) && (w.RisposteDft == risposta || w.RisposteDft == null)).Count();

                    query = $"SELECT COUNT(*) AS totaleAllegati FROM {nameof(AuditAllegatiRisposta)} g, {nameof(AuditDomandeRisposte)} h " +
                        $"WHERE g.{nameof(AuditAllegatiRisposta.IdAudit)} = h.{nameof(AuditDomandeRisposte.IdAudit)} " +
                        $"AND g.{nameof(AuditAllegatiRisposta.IdRisposta)} = h.{nameof(AuditDomandeRisposte.IdRisposta)} " +
                        $"AND (h.{nameof(AuditDomandeRisposte.Risposta)} = '{risposta}' OR h.{nameof(AuditDomandeRisposte.Risposta)} IS NULL) AND " +
                        $"h.{nameof(AuditDomandeRisposte.IdTipoDomanda)} NOT IN {listaTipiDomanda} AND h.{nameof(AuditDomandeRisposte.IdDomandaParent)} IS NULL " +
                        $"AND h.{nameof(AuditDomandeRisposte.IdAudit)} = {idAudit} AND h.{nameof(AuditDomandeRisposte.Attivo)} = 1 AND g.{nameof(AuditDomandeRisposte.Attivo)} = 1" +
                        $" AND h.{nameof(AuditDomandeRisposte.IdCategoria)} = {idCategoria}";
                    
                    int totaleAllegati = (await database.QueryAsync<int>(query)).FirstOrDefault();

                    retValue = domandeNonRisposte + totaleAllegati;
                }
            }
            catch (Exception ex)
            {
                retValue = 0;
            }

            return retValue;
        }

        public async Task<int> GetCountersAuditNc(int idAudit, int? idCategoria, int idStatoNc)
        {
            int retValue = 0;
            string query = string.Empty;

            try
            {
                List<AuditRisposteDettaglio> listaRisposteDettaglio = await database.Table<AuditRisposteDettaglio>()
                .Where(w => w.IdAudit == idAudit && w.IdStatoNC == idStatoNc && w.Attivo == true).ToListAsync();

                if (idCategoria != null)
                {
                    List<AuditDomandeRisposte> listaDomande = await database.Table<AuditDomandeRisposte>()
                        .Where(w => w.IdAudit == idAudit && w.Attivo == true && w.IdCategoria == idCategoria)
                        .ToListAsync();
                    List<int> listaIdRisposta = listaDomande.Select(s => s.IdRisposta).Distinct().ToList();

                    listaRisposteDettaglio = listaRisposteDettaglio.Where(w => listaIdRisposta.Contains(w.IdRisposta)).ToList();
                }

                retValue = listaRisposteDettaglio.Count();
            }
            catch (Exception ex)
            {
                retValue = 0;
            }

            return retValue;
        }

        public async Task<AuditFirme> GetAuditFirme(int idFirma)
        {
            return await database.FindAsync<AuditFirme>(idFirma);
        }

        public async Task<AuditFirme> GetAuditFirmeByAudit(int idAudit)
        {
            return await database.Table<AuditFirme>().Where(w => w.IdAudit == idAudit).FirstOrDefaultAsync();
        }

        public async Task<AuditSoggetti> GetAuditSoggetti(int idSoggetto)
        {
            return await database.FindAsync<AuditSoggetti>(idSoggetto);
        }

        public async Task<AuditSoggetti> GetAuditSoggettiByAudit(int idAudit)
        {
            return await database.Table<AuditSoggetti>().Where(w => w.IdAudit == idAudit).FirstOrDefaultAsync();
        }

        public async Task<AuditUtenti> GetAuditUtenti(int idUtente)
        {
            return await database.FindAsync<AuditUtenti>(idUtente);
        }

        public async Task<AuditUtenti> GetAuditUtentiByAudit(int idAudit)
        {
            return await database.Table<AuditUtenti>().Where(w => w.IdAudit == idAudit).FirstOrDefaultAsync();
        }

        public async Task<AuditPersone> GetAuditPersona(int idPersona)
        {
            return await database.FindAsync<AuditPersone>(idPersona);
        }

        public async Task<List<AuditPersone>> GetAuditPersone(int idAudit)
        {
            return await database.Table<AuditPersone>().Where(w => w.IdAudit == idAudit).ToListAsync();
        }

        public async Task<AuditPersone> GetAuditPersone(int idAudit, int idPersona)
        {
            return await database.Table<AuditPersone>().Where(w => w.IdAudit == idAudit && w.IdPersona == idPersona).FirstOrDefaultAsync();
        }

        public async Task<List<AuditDomandeRisposte>> GetAuditDomande(int idAudit, int idCategoria)
        {
            return await database.Table<AuditDomandeRisposte>().Where(w => w.IdAudit == idAudit && w.IdCategoria == idCategoria)
                //.OrderBy(o => o.IdDomanda).ThenBy(t => t.Testo).ThenBy(t => t.IdDomandaParent)
                .OrderByDescending(o => o.IdDomanda)
                .ToListAsync();
        }

        public async Task<AuditDomandeRisposte> GetAuditDomanda(int pkid)
        {
            return await database.Table<AuditDomandeRisposte>().Where(w => w.PKID == pkid).FirstOrDefaultAsync();
        }

        public async Task<AuditDomandeRisposte> GetAuditDomandaFromParent(int idAudit, int? idDomandaParent)
        {
            return await database.Table<AuditDomandeRisposte>().Where(w => w.IdAudit == idAudit && w.IdDomanda == idDomandaParent).FirstOrDefaultAsync();
        }

        public async Task<int> ContaDettagliDomanda(int idAudit, int idRisposta)
        {
            return await database.Table<AuditRisposteDettaglio>().Where(w => w.IdAudit == idAudit && w.IdRisposta == idRisposta && w.Attivo == true).CountAsync();
        }

        public async Task<int> ContaAllegatiDomanda(int idAudit, int idRisposta)
        {
            int retValue = 0;

            retValue += await database.Table<AuditAllegatiRisposta>().Where(w => w.IdAudit == idAudit && w.IdRisposta == idRisposta && w.IdRispostaDettaglio == null && w.Attivo == true).CountAsync();
            retValue += await database.Table<AuditGrigliaAllegati>().Where(w => w.IdAudit == idAudit && w.IdRisposta == idRisposta && w.IdRispostaDettaglio == null && w.Attivo == true).CountAsync();
            retValue += await database.Table<AuditDocumentiEsaminati>().Where(w => w.IdAudit == idAudit && w.IdRisposta == idRisposta && w.IdRispostaDettaglio == null && w.Attivo == true).CountAsync();

            return retValue;
        }

        public async Task<int> UpdateAudit(Audit audit)
        {
            Audit oldAudit = await GetAudit(audit.IdAudit);
            int retValue = 0;

            if (oldAudit != null)
            {
                oldAudit.CodAudit = audit.CodAudit;
                oldAudit.DescAudit = audit.DescAudit;
                oldAudit.DescAudit2 = audit.DescAudit2;
                oldAudit.ColoreTipoAudit = audit.ColoreTipoAudit;
                oldAudit.DataInizioAudit = audit.DataInizioAudit;
                oldAudit.DataFineAudit = audit.DataFineAudit;
                oldAudit.OraInizio = audit.OraInizio;
                oldAudit.OraFine = audit.OraFine;
                oldAudit.IdStato = audit.IdStato;
                oldAudit.IdSoggetto = audit.IdSoggetto;
                oldAudit.DesSoggetto = audit.DesSoggetto;
                oldAudit.Immagine = audit.Immagine;
                oldAudit.Etichetta1 = audit.Etichetta1;
                oldAudit.ValEtichetta1 = audit.ValEtichetta1;
                oldAudit.Etichetta2 = audit.Etichetta2;
                oldAudit.ValEtichetta2 = audit.ValEtichetta2;
                oldAudit.Etichetta3 = audit.Etichetta3;
                oldAudit.ValEtichetta3 = audit.ValEtichetta3;
                oldAudit.Etichetta4 = audit.Etichetta4;
                oldAudit.ValEtichetta4 = audit.ValEtichetta4;
                oldAudit.Etichetta5 = audit.Etichetta5;
                oldAudit.ValEtichetta5 = audit.ValEtichetta5;
                oldAudit.Etichetta6 = audit.Etichetta6;
                oldAudit.ValEtichetta6 = audit.ValEtichetta6;
                oldAudit.Etichetta7 = audit.Etichetta7;
                oldAudit.ValEtichetta7 = audit.ValEtichetta7;
                oldAudit.Etichetta8 = audit.Etichetta8;
                oldAudit.ValEtichetta8 = audit.ValEtichetta8;
                oldAudit.Etichetta9 = audit.Etichetta9;
                oldAudit.ValEtichetta9 = audit.ValEtichetta9;
                oldAudit.Etichetta10 = audit.Etichetta10;
                oldAudit.ValEtichetta10 = audit.ValEtichetta10;
                oldAudit.Etichetta11 = audit.Etichetta11;
                oldAudit.ValEtichetta11 = audit.ValEtichetta11;
                oldAudit.Etichetta12 = audit.Etichetta12;
                oldAudit.ValEtichetta12 = audit.ValEtichetta12;
                oldAudit.Etichetta13 = audit.Etichetta13;
                oldAudit.ValEtichetta13 = audit.ValEtichetta13;
                oldAudit.Etichetta14 = audit.Etichetta14;
                oldAudit.ValEtichetta14 = audit.ValEtichetta14;
                oldAudit.Etichetta15 = audit.Etichetta15;
                oldAudit.ValEtichetta15 = audit.ValEtichetta15;
                oldAudit.Etichetta16 = audit.Etichetta16;
                oldAudit.ValEtichetta16 = audit.ValEtichetta16;
                oldAudit.Etichetta17 = audit.Etichetta17;
                oldAudit.ValEtichetta17 = audit.ValEtichetta17;
                oldAudit.MailAudit = audit.MailAudit;
                oldAudit.DataProgress = audit.DataProgress;
                oldAudit.SogliaSx = audit.SogliaSx;
                oldAudit.SogliaDx = audit.SogliaDx;
                oldAudit.SogliaPrua = audit.SogliaPrua;
                oldAudit.SogliaPoppa = audit.SogliaPoppa;
                oldAudit.MacroNote1 = audit.MacroNote1;
                oldAudit.AttributoNote1 = audit.AttributoNote1;
                oldAudit.MacroNote2 = audit.MacroNote2;
                oldAudit.AttributoNote2 = audit.AttributoNote2;
                oldAudit.MacroNote3 = audit.MacroNote3;
                oldAudit.AttributoNote3 = audit.AttributoNote3;
                oldAudit.MacroNote4 = audit.MacroNote4;
                oldAudit.AttributoNote4 = audit.AttributoNote4;
                oldAudit.MacroNote5 = audit.MacroNote5;
                oldAudit.AttributoNote5 = audit.AttributoNote5;
                oldAudit.MacroNote6 = audit.MacroNote6;
                oldAudit.AttributoNote6 = audit.AttributoNote6;
                oldAudit.MacroNote7 = audit.MacroNote7;
                oldAudit.AttributoNote7 = audit.AttributoNote7;
                oldAudit.MacroNote8 = audit.MacroNote8;
                oldAudit.AttributoNote8 = audit.AttributoNote8;
                oldAudit.MacroNote9 = audit.MacroNote9;
                oldAudit.AttributoNote9 = audit.AttributoNote9;
                oldAudit.MacroNote10 = audit.MacroNote10;
                oldAudit.AttributoNote10 = audit.AttributoNote10;
                oldAudit.MacroNote11 = audit.MacroNote11;
                oldAudit.AttributoNote11 = audit.AttributoNote11;
                oldAudit.MacroNote12 = audit.MacroNote12;
                oldAudit.AttributoNote12 = audit.AttributoNote12;
                oldAudit.CreatoInApp = audit.CreatoInApp;
                oldAudit.DataCreatoInApp = audit.DataCreatoInApp;
                oldAudit.IdAuditApp = audit.IdAuditApp;
                oldAudit.NrFirme = audit.NrFirme;
                oldAudit.PlaceholderFirme = audit.PlaceholderFirme;
                oldAudit.BloccoApp1 = audit.BloccoApp1;
                oldAudit.BloccoApp2 = audit.BloccoApp2;
                oldAudit.BloccoApp3 = audit.BloccoApp3;
                oldAudit.FiltroSoggetto = audit.FiltroSoggetto;
                oldAudit.IconaTipoAudit = audit.IconaTipoAudit;

                retValue = await database.UpdateAsync(oldAudit);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditFirme(AuditFirme auditFirme)
        {
            AuditFirme oldAuditFirme = await GetAuditFirme(auditFirme.PKID);
            int retValue = 0;

            if (oldAuditFirme != null)
            {
                oldAuditFirme.NomeFirma = auditFirme.NomeFirma;
                oldAuditFirme.FirmaSoggetto = auditFirme.FirmaSoggetto;
                oldAuditFirme.IdAudit = auditFirme.IdAudit;
                oldAuditFirme.Sincronizzato = auditFirme.Sincronizzato;
                oldAuditFirme.DataCancellazione = auditFirme.DataCancellazione;
                oldAuditFirme.DataCreazione = auditFirme.DataCreazione;
                oldAuditFirme.DataSincronizzato = auditFirme.DataSincronizzato;
                oldAuditFirme.Attivo = auditFirme.Attivo;

                retValue = await database.UpdateAsync(oldAuditFirme);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditSoggetti(AuditSoggetti auditSoggetto)
        {
            AuditSoggetti oldAuditSoggetto = await GetAuditSoggetti(auditSoggetto.PKID);
            int retValue = 0;

            if (oldAuditSoggetto != null)
            {
                oldAuditSoggetto.IdAudit = auditSoggetto.IdAudit;
                oldAuditSoggetto.IdSoggetto = auditSoggetto.IdSoggetto;
                oldAuditSoggetto.DescSoggetto = auditSoggetto.DescSoggetto;
                oldAuditSoggetto.IdTipoSoggetto = auditSoggetto.IdTipoSoggetto;
                oldAuditSoggetto.Matricola = auditSoggetto.Matricola;
                oldAuditSoggetto.Localita = auditSoggetto.Localita;
                oldAuditSoggetto.Email = auditSoggetto.Email;
                oldAuditSoggetto.Telefono = auditSoggetto.Telefono;
                oldAuditSoggetto.Cellulare = auditSoggetto.Cellulare;
                oldAuditSoggetto.Note = auditSoggetto.Note;
                oldAuditSoggetto.Divisione = auditSoggetto.Divisione;
                oldAuditSoggetto.AttributoSoggetto1 = auditSoggetto.AttributoSoggetto1;
                oldAuditSoggetto.AttributoSoggetto2 = auditSoggetto.AttributoSoggetto2;
                oldAuditSoggetto.AttributoSoggetto3 = auditSoggetto.AttributoSoggetto3;
                oldAuditSoggetto.AttributoSoggetto4 = auditSoggetto.AttributoSoggetto4;
                oldAuditSoggetto.Immagine = auditSoggetto.Immagine;
                oldAuditSoggetto.CodProdotto = auditSoggetto.CodProdotto;
                oldAuditSoggetto.Filtro = auditSoggetto.Filtro;

                retValue = await database.UpdateAsync(oldAuditSoggetto);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditCategoria(AuditCategorie auditCategoria)
        {
            AuditCategorie oldAuditCategoria = await GetAuditCategoria(auditCategoria.PKID);
            int retValue = 0;

            if (oldAuditCategoria != null)
            {
                oldAuditCategoria.IdAudit = auditCategoria.IdAudit;
                oldAuditCategoria.IdCategoria = auditCategoria.IdCategoria;
                oldAuditCategoria.CodCategoria = auditCategoria.CodCategoria;
                oldAuditCategoria.DescCategoria = auditCategoria.DescCategoria;
                oldAuditCategoria.Immagine = auditCategoria.Immagine;
                oldAuditCategoria.Readonly = auditCategoria.Readonly;
                oldAuditCategoria.IdTipoAudit = auditCategoria.IdTipoAudit;

                retValue = await database.UpdateAsync(oldAuditCategoria);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditChecklist(AuditChecklist auditChecklist)
        {
            AuditChecklist oldAuditChecklist = await GetAuditChecklist(auditChecklist.PKID);
            int retValue = 0;

            if (oldAuditChecklist != null)
            {
                oldAuditChecklist.IdAudit = auditChecklist.IdAudit;
                oldAuditChecklist.IdCheckList = auditChecklist.IdCheckList;
                oldAuditChecklist.CodCheckList = auditChecklist.CodCheckList;
                oldAuditChecklist.DescChecklist = auditChecklist.DescChecklist;
                oldAuditChecklist.DescChecklist2 = auditChecklist.DescChecklist2;
                oldAuditChecklist.Immagine = auditChecklist.Immagine;
                oldAuditChecklist.IdTipoAudit = auditChecklist.IdTipoAudit;

                retValue = await database.UpdateAsync(oldAuditChecklist);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditUtenti(AuditUtenti auditUtenti)
        {
            AuditUtenti oldAuditUtenti = await GetAuditUtenti(auditUtenti.PKID);
            int retValue = 0;

            if (oldAuditUtenti != null)
            {
                oldAuditUtenti.IdAudit = auditUtenti.IdAudit;
                oldAuditUtenti.IdUtente = auditUtenti.IdUtente;
                oldAuditUtenti.Nome = auditUtenti.Nome;
                oldAuditUtenti.Cognome = auditUtenti.Cognome;
                oldAuditUtenti.UserCode = auditUtenti.UserCode;
                oldAuditUtenti.Email = auditUtenti.Email;
                oldAuditUtenti.Telefono = auditUtenti.Telefono;
                oldAuditUtenti.Cellulare = auditUtenti.Cellulare;
                oldAuditUtenti.Immagine = auditUtenti.Immagine;
                oldAuditUtenti.Firma = auditUtenti.Firma;
                oldAuditUtenti.EnumTema = auditUtenti.EnumTema;

                retValue = await database.UpdateAsync(oldAuditUtenti);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditPersone(AuditPersone auditPersone)
        {
            AuditPersone oldAuditPersone = await GetAuditPersona(auditPersone.PKID);
            int retValue = 0;

            if (oldAuditPersone != null)
            {
                oldAuditPersone.IdPersona = auditPersone.IdPersona;
                oldAuditPersone.IdAudit = auditPersone.IdAudit;
                oldAuditPersone.Nome = auditPersone.Nome;
                oldAuditPersone.Cognome = auditPersone.Cognome;
                oldAuditPersone.Telefono = auditPersone.Telefono;
                oldAuditPersone.Mail = auditPersone.Mail;
                oldAuditPersone.Immagine = auditPersone.Immagine;
                oldAuditPersone.Firma = auditPersone.Firma;
                oldAuditPersone.Messaggio = auditPersone.Messaggio;

                retValue = await database.UpdateAsync(oldAuditPersone);
            }

            return retValue;
        }

        public async Task<int> UpdateAuditDomandeRisposte(AuditDomandeRisposte auditDomandeRisposte)
        {
            AuditDomandeRisposte oldAuditDomandeRisposte = await GetAuditDomanda(auditDomandeRisposte.PKID);
            int retValue = 0;

            if (oldAuditDomandeRisposte != null)
            {
                oldAuditDomandeRisposte.IdDomanda = auditDomandeRisposte.IdDomanda;
                oldAuditDomandeRisposte.IdTipoDomanda = auditDomandeRisposte.IdTipoDomanda;
                oldAuditDomandeRisposte.IdCategoria = auditDomandeRisposte.IdCategoria;
                oldAuditDomandeRisposte.Testo = auditDomandeRisposte.Testo;
                oldAuditDomandeRisposte.Valori = auditDomandeRisposte.Valori;
                oldAuditDomandeRisposte.Help = auditDomandeRisposte.Help;
                oldAuditDomandeRisposte.HelpAllegato = auditDomandeRisposte.HelpAllegato;
                oldAuditDomandeRisposte.ValoreDomandaParent = auditDomandeRisposte.ValoreDomandaParent;
                oldAuditDomandeRisposte.IdCriticita = auditDomandeRisposte.IdCriticita;
                oldAuditDomandeRisposte.Attributo = auditDomandeRisposte.Attributo;
                oldAuditDomandeRisposte.Punteggio = auditDomandeRisposte.Punteggio;
                oldAuditDomandeRisposte.CodPuntoNorma = auditDomandeRisposte.CodPuntoNorma;
                oldAuditDomandeRisposte.DesPuntoNorma = auditDomandeRisposte.DesPuntoNorma;
                oldAuditDomandeRisposte.IdProcessoInterno = auditDomandeRisposte.IdProcessoInterno;
                oldAuditDomandeRisposte.RisposteNcDft = auditDomandeRisposte.RisposteNcDft;
                oldAuditDomandeRisposte.NrCampiNote = auditDomandeRisposte.NrCampiNote;
                oldAuditDomandeRisposte.PlaceholderNote = auditDomandeRisposte.PlaceholderNote;
                oldAuditDomandeRisposte.TipoCampiNote = auditDomandeRisposte.TipoCampiNote;
                oldAuditDomandeRisposte.RisposteDft = auditDomandeRisposte.RisposteDft;
                oldAuditDomandeRisposte.IdRisposteDettaglioMacroAttributo1 = auditDomandeRisposte.IdRisposteDettaglioMacroAttributo1;
                oldAuditDomandeRisposte.IdRisposteDettaglioAttributo1 = auditDomandeRisposte.IdRisposteDettaglioAttributo1;
                oldAuditDomandeRisposte.IdRisposteDettaglioMacroAttributo2 = auditDomandeRisposte.IdRisposteDettaglioMacroAttributo2;
                oldAuditDomandeRisposte.IdRisposteDettaglioAttributo2 = auditDomandeRisposte.IdRisposteDettaglioAttributo2;
                oldAuditDomandeRisposte.IdRisposteDettaglioMacroAttributo3 = auditDomandeRisposte.IdRisposteDettaglioMacroAttributo3;
                oldAuditDomandeRisposte.IdRisposteDettaglioAttributo3 = auditDomandeRisposte.IdRisposteDettaglioAttributo3;
                oldAuditDomandeRisposte.IdRisposteDettaglioMacroAttributo4 = auditDomandeRisposte.IdRisposteDettaglioMacroAttributo4;
                oldAuditDomandeRisposte.IdRisposteDettaglioAttributo4 = auditDomandeRisposte.IdRisposteDettaglioAttributo4;
                oldAuditDomandeRisposte.IdRisposteDettaglioMacroAttributo5 = auditDomandeRisposte.IdRisposteDettaglioMacroAttributo5;
                oldAuditDomandeRisposte.IdRisposteDettaglioAttributo5 = auditDomandeRisposte.IdRisposteDettaglioAttributo5;
                oldAuditDomandeRisposte.IdTipoAudit = auditDomandeRisposte.IdTipoAudit;
                oldAuditDomandeRisposte.IdChecklist = auditDomandeRisposte.IdChecklist;
                oldAuditDomandeRisposte.DesTipoDomande = auditDomandeRisposte.DesTipoDomande;
                oldAuditDomandeRisposte.PrevedeProcesso = auditDomandeRisposte.PrevedeProcesso;
                oldAuditDomandeRisposte.IdProcesso = auditDomandeRisposte.IdProcesso;
                oldAuditDomandeRisposte.IdAudit = auditDomandeRisposte.IdAudit;
                oldAuditDomandeRisposte.DataCreazione = auditDomandeRisposte.DataCreazione;
                oldAuditDomandeRisposte.Attivo = auditDomandeRisposte.Attivo;
                oldAuditDomandeRisposte.DataCancellazione = auditDomandeRisposte.DataCancellazione;
                oldAuditDomandeRisposte.IdRisposta = auditDomandeRisposte.IdRisposta;
                oldAuditDomandeRisposte.DomandaCustom = auditDomandeRisposte.DomandaCustom;
                oldAuditDomandeRisposte.SincronizzatoCustom = auditDomandeRisposte.SincronizzatoCustom;
                oldAuditDomandeRisposte.DataSincronizzatoCustom = auditDomandeRisposte.DataSincronizzatoCustom;

                oldAuditDomandeRisposte.Risposta = auditDomandeRisposte.Risposta;
                oldAuditDomandeRisposte.Note = auditDomandeRisposte.Note;

                oldAuditDomandeRisposte.Sincronizzato = false;
                oldAuditDomandeRisposte.DataSincronizzato = null;
                oldAuditDomandeRisposte.DataRisposta = DateTime.Now;

                retValue = await database.UpdateAsync(oldAuditDomandeRisposte);
            }

            return retValue;
        }
        #endregion

        #region Soggetti
        public async Task<List<Soggetti>> GetSoggetti()
        {
            return await database.Table<Soggetti>().OrderBy(o => o.DescSoggetto).ToListAsync();
        }

        public async Task<List<Soggetti>> GetSoggettiFilterAsync()
        {
            return await database.QueryAsync<Soggetti>("SELECT Filtro, GROUP_CONCAT(IdSoggetto) AS IdSoggetto FROM Soggetti GROUP BY Filtro ORDER BY Filtro");
        }
        #endregion

        #region Categorie
        public async Task<List<Categorie>> GetCategorie()
        {
            return await database.Table<Categorie>().OrderBy(o => o.DescCategoria).ToListAsync();
        }
        #endregion

        #region Stati
        public async Task<List<Stati>> GetStati()
        {
            return await database.Table<Stati>().OrderBy(o => o.IdStato).ToListAsync();
        }

        public async Task<List<StatiDomandeRisposteDettaglio>> GetStatiNC()
        {
            return await database.Table<StatiDomandeRisposteDettaglio>().OrderBy(o => o.IdStatoNC).ToListAsync();
        }
        #endregion

        #region Tipi Audit
        public async Task<int> ContaTipiAudit()
        {
            return await database.Table<TipiAudit>().CountAsync();
        }

        public async Task<List<TipiAudit>> GetTipiAudit()
        {
            return await database.Table<TipiAudit>().OrderBy(o => o.DesTipoAudit).ToListAsync();
        }
        #endregion

        #region Motivazioni
        public async Task<List<Motivazioni>> GetMotivazioni()
        {
            return await database.Table<Motivazioni>().OrderBy(o => o.DescMotivazione).ToListAsync();
        }
        #endregion

        #region Utenti
        public async Task<int> GetUserId()
        {
            return (await database.Table<Utenti>().Where(w => w.Email == Settings.User).FirstOrDefaultAsync()).IdUtente;
        }
        #endregion


        #region Common
        public async Task<int> ContaElementiNonSincronizzatiAsync(int idAudit)
        {
            int contaDomandeCustomNonSincronizzate = await database.Table<AuditDomandeRisposte>().Where(w => w.IdAudit == idAudit && w.SincronizzatoCustom == false).CountAsync();
            int contaDomandeNonSincronizzate = await database.Table<AuditDomandeRisposte>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            int contaDettagliNonSincronizzati = await database.Table<AuditRisposteDettaglio>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            int contaAllegatiNonSincronizzati = await database.Table<AuditAllegatiRisposta>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            int contaRisposteFirmeNonSincronizzate = await database.Table<AuditRisposteFirme>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            int contaVersioniNonSincronizzate = await database.Table<AuditDocumentiEsaminati>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            int contaGriglieAllegatiNonSincronizzati = await database.Table<AuditGrigliaAllegati>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();
            //int contaRegistrazioniPiattaformaInerzialeNonSincronizzate = database.Table<RegistrazioniPiattaformaInerziale>().Where(w => w.idAudit == idAuditInt && w.Sincronizzato == 1).Count();
            //int contaOrariLavoroNonSincronizzati = database.Table<ContaLavoro>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == 1).Count();
            //int contaAuditCompletatiNonSincronizzati = database.Table<AuditCompletato>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == "0").Count();
            int contaAuditFirmeNonSincronizzate = await database.Table<AuditFirme>().Where(w => w.IdAudit == idAudit && w.Sincronizzato == false).CountAsync();

            int sommaConteggioNonSincronizzati = contaDomandeCustomNonSincronizzate +
                contaDomandeNonSincronizzate + contaDettagliNonSincronizzati +
                contaAllegatiNonSincronizzati + contaRisposteFirmeNonSincronizzate +
                contaVersioniNonSincronizzate + contaGriglieAllegatiNonSincronizzati +
                //contaRegistrazioniPiattaformaInerzialeNonSincronizzate + contaOrariLavoroNonSincronizzati + contaAuditCompletatiNonSincronizzati +
                contaAuditFirmeNonSincronizzate;

            return sommaConteggioNonSincronizzati;
        }

        public async Task DeleteCascadeAsync(int idAudit)
        {
            string queryDelete = $"DELETE FROM Tabella WHERE Campo = {idAudit}";
            await database.QueryAsync<AuditTipiAudit>(queryDelete.Replace("Tabella", nameof(AuditTipiAudit)).Replace("Campo", nameof(AuditTipiAudit.IdAudit)));
            await database.QueryAsync<AuditUtenti>(queryDelete.Replace("Tabella", nameof(AuditUtenti)).Replace("Campo", nameof(AuditUtenti.IdAudit)));
            await database.QueryAsync<AuditMotivi>(queryDelete.Replace("Tabella", nameof(AuditMotivi)).Replace("Campo", nameof(AuditMotivi.IdAudit)));
            await database.QueryAsync<AuditChecklist>(queryDelete.Replace("Tabella", nameof(AuditChecklist)).Replace("Campo", nameof(AuditChecklist.IdAudit)));
            await database.QueryAsync<AuditCategorie>(queryDelete.Replace("Tabella", nameof(AuditCategorie)).Replace("Campo", nameof(AuditCategorie.IdAudit)));
            await database.QueryAsync<AuditCheckCategorie>(queryDelete.Replace("Tabella", nameof(AuditCheckCategorie)).Replace("Campo", nameof(AuditCheckCategorie.IdAudit)));
            await database.QueryAsync<AuditAllegati>(queryDelete.Replace("Tabella", nameof(AuditAllegati)).Replace("Campo", nameof(AuditAllegati.IdAudit)));
            await database.QueryAsync<AuditPersone>(queryDelete.Replace("Tabella", nameof(AuditPersone)).Replace("Campo", nameof(AuditPersone.IdAudit)));
            await database.QueryAsync<AuditServizi>(queryDelete.Replace("Tabella", nameof(AuditServizi)).Replace("Campo", nameof(AuditServizi.IdAudit)));
            await database.QueryAsync<AuditSoggetti>(queryDelete.Replace("Tabella", nameof(AuditSoggetti)).Replace("Campo", nameof(AuditSoggetti.IdAudit)));
            await database.QueryAsync<AuditDomandeRisposte>(queryDelete.Replace("Tabella", nameof(AuditDomandeRisposte)).Replace("Campo", nameof(AuditDomandeRisposte.IdAudit)));
            await database.QueryAsync<AuditRisposteDettaglio>(queryDelete.Replace("Tabella", nameof(AuditRisposteDettaglio)).Replace("Campo", nameof(AuditRisposteDettaglio.IdAudit)));
            await database.QueryAsync<AuditAllegatiRisposta>(queryDelete.Replace("Tabella", nameof(AuditAllegatiRisposta)).Replace("Campo", nameof(AuditAllegatiRisposta.IdAudit)));
            await database.QueryAsync<AuditFirme>(queryDelete.Replace("Tabella", nameof(AuditFirme)).Replace("Campo", nameof(AuditFirme.IdAudit)));
            await database.QueryAsync<AuditAuditor>(queryDelete.Replace("Tabella", nameof(AuditAuditor)).Replace("Campo", nameof(AuditAuditor.IdAudit)));
            await database.QueryAsync<AuditDomandeRelated>(queryDelete.Replace("Tabella", nameof(AuditDomandeRelated)).Replace("Campo", nameof(AuditDomandeRelated.IdAudit)));
            //await database.QueryAsync<AuditContaLavoro>(queryDelete.Replace("Tabella", nameof(AuditContaLavoro)).Replace("Campo", nameof(AuditContaLavoro.IdAudit)));
            await database.QueryAsync<AuditRisposteFirme>(queryDelete.Replace("Tabella", nameof(AuditRisposteFirme)).Replace("Campo", nameof(AuditRisposteFirme.IdAudit)));
            //await database.QueryAsync<AuditDisassociato>(queryDelete.Replace("Tabella", nameof(AuditDisassociato)).Replace("Campo", nameof(AuditDisassociato.IdAudit)));
            //await database.QueryAsync<AuditCompletato>(queryDelete.Replace("Tabella", nameof(AuditCompletato)).Replace("Campo", nameof(AuditCompletato.IdAudit)));
            //await database.QueryAsync<RegistrazioniPiattaformaInerziale>(queryDelete.Replace("Tabella", nameof(RegistrazioniPiattaformaInerziale)).Replace("Campo", nameof(RegistrazioniPiattaformaInerziale.IdAudit))});
            await database.QueryAsync<AuditGrigliaAllegati>(queryDelete.Replace("Tabella", nameof(AuditGrigliaAllegati)).Replace("Campo", nameof(AuditGrigliaAllegati.IdAudit)));
            await database.QueryAsync<AuditDocumentiEsaminati>(queryDelete.Replace("Tabella", nameof(AuditDocumentiEsaminati)).Replace("Campo", nameof(AuditDocumentiEsaminati.IdAudit)));
        }

        public async Task<bool> GetFunzioniApp(int idFunzioneApp, int? idTipoaudit)
        {
            int idUtente = await GetUserId();
            List<UtentiFunzioniApp> listaFunzioni = await database.Table<UtentiFunzioniApp>().Where(w => w.IdUtente == idUtente && w.IdFunzioneApp == idFunzioneApp).ToListAsync();

            if (idTipoaudit != null)
                listaFunzioni = listaFunzioni.Where(w => w.IdTipoAudit == idTipoaudit).ToList();

            return (listaFunzioni != null && listaFunzioni.Count > 0);
        }

        public async Task<List<ValoriRisposta>> GetValoriRisposta()
        {
            return await database.Table<ValoriRisposta>().ToListAsync();
        }

        public async Task<List<DatiIndice>> GetChartData(int idAudit)
        {
            string queryFromWhere = $"FROM {nameof(AuditRisposteDettaglio)} a, {nameof(Indici)} b, {nameof(RisposteDettaglioAttributo)} c " +
                $"WHERE a.{nameof(AuditRisposteDettaglio.IdRisposteDettaglioAttributo2)} = b.{nameof(Indici.IdRisposteDettaglioAttributo)} " +
                $"AND a.{nameof(AuditRisposteDettaglio.IdRisposteDettaglioAttributo3)} = c.{nameof(RisposteDettaglioAttributo.IdRisposteDettaglioAttributo)} " +
                $"AND a.{nameof(AuditRisposteDettaglio.IdAudit)} = {idAudit} " +
                $"AND a.{nameof(AuditRisposteDettaglio.Attivo)} = 1 AND a.{nameof(AuditRisposteDettaglio.IdStatoNC)} <> 3";

            string query = $"SELECT COALESCE(SUM(c.{nameof(RisposteDettaglioAttributo.PesoAttributoDomande)}), 0) AS Valore, b.{nameof(Indici.CodIndice)} AS Codice, b.{nameof(Indici.DescIndice)} AS Descrizione, b.{nameof(Indici.ColoreIndice)}, " +
                $"COUNT(*) AS Conteggio {queryFromWhere} GROUP BY b.{nameof(Indici.CodIndice)}, b.{nameof(Indici.DescIndice)}, b.{nameof(Indici.ColoreIndice)} ORDER BY b.{nameof(Indici.CodIndice)}";

            List<DatiIndice> items = await database.QueryAsync<DatiIndice>(query);

            return items;
        }
        #endregion
    }
}