﻿using AuditManager.Models.Tables;
using AuditManager.Shared.Constants;
using AuditManager.Shared.Types;
using AuditManager.Views;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net.Http;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace AuditManager.Helpers
{
    public class FunctionsHelper
    {
        public FunctionsHelper()
        {

        }

        public static Color GetColorFromResource(string resourceName)
        {
            if (!string.IsNullOrEmpty(resourceName))
                return (Color)Application.Current.Resources[resourceName];

            return Color.Transparent;
        }

        public static async Task Log(string operazione, string messaggio, bool insertDb = true)
        {
            Debug.WriteLine($"LOG: {operazione} - {messaggio}");

            if (insertDb)
            {
                Log log = new Log
                {
                    DataOperazione = DateTime.Now,
                    DeviceOperazione = Settings.DeviceId,
                    EsitoOperazione = false,
                    ExceptionMessage = messaggio,
                    Operazione = operazione,
                    UtenteOperazione = Settings.User
                };

                await App.Database.InsertLogAsync(log);
            }
        }

        public static ImageSource SetImageSource(string pathImage)
        {
            ImageSource retValue = null;

            if (Device.RuntimePlatform == Device.UWP)
                retValue = ImageSource.FromFile($"Images\\{pathImage}");
            else
                retValue = ImageSource.FromFile(pathImage);

            return retValue;
        }

        public static string SetImageString(string pathImage)
        {
            string retValue = pathImage;

            if (Device.RuntimePlatform == Device.UWP)
                retValue = $"Images\\{pathImage}";

            return retValue;
        }

        public static void Logout()
        {
            Settings.UserName = string.Empty;
            Settings.Password = string.Empty;
            Settings.User = string.Empty;
            Settings.IsOnline = true;
            Settings.UserImage = string.Empty;

            Application.Current.MainPage = new NavigationPage(new LoginPage());
        }

        public static T Clone<T>(T source)
        {
            if (!typeof(T).IsSerializable)
                throw new ArgumentException("The type must be serializable.", "source");

            if (ReferenceEquals(source, null))
                return default(T);

            IFormatter formatter = new BinaryFormatter();
            Stream stream = new MemoryStream();
            using (stream)
            {
                try
                {
                    formatter.Serialize(stream, source);
                    stream.Seek(0, SeekOrigin.Begin);
                    return (T)formatter.Deserialize(stream);
                }
                catch (Exception ex)
                {
                    if (ReferenceEquals(source, null))
                        return default(T);

                    var deserializeSettings = new JsonSerializerSettings { ObjectCreationHandling = ObjectCreationHandling.Replace };
                    return JsonConvert.DeserializeObject<T>(JsonConvert.SerializeObject(source), deserializeSettings);
                }
            }
        }

        public static async Task DownloadImagesAudit(int idAudit)
        {
            bool retValue = true;
            Audit a = await App.Database.GetAudit(idAudit);

            if (!a.ImmagineScaricata)
                retValue = await DownloadImagesAudit(idAudit, 0, a.Immagine, Images.Soggetti);

            if (retValue)
            {
                List<AuditCategorie> categorie = await App.Database.GetAuditCategorie(idAudit);

                foreach (var item in categorie)
                {
                    retValue = await DownloadImagesAudit(idAudit, item.IdCategoria, item.Immagine, Images.Categorie);
                }

                AuditFirme firma = await App.Database.GetAuditFirmeByAudit(idAudit);

                if (firma != null)
                    retValue = await DownloadImagesAudit(idAudit, 0, firma.FirmaSoggetto, Images.FirmaAudit);

                AuditUtenti utenti = await App.Database.GetAuditUtentiByAudit(idAudit);

                if (utenti != null)
                {
                    retValue = await DownloadImagesAudit(idAudit, utenti.IdUtente, utenti.Immagine, Images.Utenti);
                    retValue = await DownloadImagesAudit(idAudit, utenti.IdUtente, utenti.Firma, Images.Firme);
                }

                AuditChecklist checklist = await App.Database.GetAuditChecklist(idAudit);

                if (checklist != null)
                    retValue = await DownloadImagesAudit(idAudit, checklist.IdCheckList, checklist.Immagine, Images.Checklist);

                List<AuditPersone> persone = await App.Database.GetAuditPersone(idAudit);

                foreach (var item in persone)
                {
                    retValue = await DownloadImagesAudit(idAudit, item.IdPersona, item.Immagine, Images.Persone);
                }
            }
        }

        public static async Task<bool> DownloadImagesAudit(int idAudit, int chiaveTabella, string image, Images tipoImmagine)
        {
            bool doCreate = false;
            bool okUpdate = false;
            bool retValue = true;

            string folder = string.Empty;
            string immagine = string.Empty;
            string immagineReturn = string.Empty;
            string filePath = string.Empty;

            if (!string.IsNullOrEmpty(image))
            {
                if (image.IndexOf(Constants.RootUrl) >= 0)
                {
                    immagine = image.Replace($"{Constants.RootUrl}/Images/", "");

                    string[] split = immagine.Split(new[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
                    folder = split[0].ToUpper();
                    immagine = split[1];

                    switch (tipoImmagine)
                    {
                        case Images.Soggetti:
                        case Images.Categorie:
                        case Images.FirmaAudit:
                        case Images.Checklist:
                        case Images.Icone:
                            folder = $"{folder}_{idAudit}";
                            break;
                        case Images.Utenti:
                        case Images.Persone:
                        case Images.Firme:
                        default:
                            break;
                    }

                    try
                    {
                        string folderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), folder);

                        if (!Directory.Exists(folderPath))
                            Directory.CreateDirectory(folderPath);

                        filePath = Path.Combine(folderPath, immagine);

                        doCreate = !File.Exists(filePath);

                        if (doCreate)
                        {
                            try
                            {
                                HttpClient client = new HttpClient();
                                HttpResponseMessage message = await client.GetAsync(image);
                                byte[] file = await message.Content.ReadAsByteArrayAsync();

                                File.WriteAllBytes(filePath, file);
                                okUpdate = true;
                            }
                            catch (Exception ex)
                            {
                                await Log(nameof(DownloadImagesAudit), ex.Message);
                                okUpdate = false;
                            }
                        }
                        else
                            okUpdate = true;
                    }
                    catch (Exception ex)
                    {
                        await Log(nameof(DownloadImagesAudit), ex.Message);
                        okUpdate = false;
                    }
                }
                else
                    okUpdate = false;
            }
            else
                okUpdate = false;

            if (okUpdate)
            {
                immagineReturn = filePath;

                try
                {
                    switch (tipoImmagine)
                    {
                        case Images.Soggetti:
                            Audit a = await App.Database.GetAudit(idAudit);
                            a.Immagine = immagineReturn;
                            await App.Database.UpdateAudit(a);

                            AuditSoggetti auditSoggetti = await App.Database.GetAuditSoggettiByAudit(idAudit);
                            auditSoggetti.Immagine = immagineReturn;
                            await App.Database.UpdateAuditSoggetti(auditSoggetti);

                            break;
                        case Images.Categorie:
                            AuditCategorie auditCategorie = await App.Database.GetAuditCategoria(idAudit, chiaveTabella);
                            auditCategorie.Immagine = immagineReturn;
                            await App.Database.UpdateAuditCategoria(auditCategorie);

                            break;
                        case Images.FirmaAudit:
                            AuditFirme af = await App.Database.GetAuditFirmeByAudit(idAudit);
                            af.FirmaSoggetto = immagineReturn;
                            await App.Database.UpdateAuditFirme(af);

                            break;
                        case Images.Firme:
                            AuditUtenti auditUtentiFirma = await App.Database.GetAuditUtentiByAudit(idAudit);
                            auditUtentiFirma.Firma = immagineReturn;
                            await App.Database.UpdateAuditUtenti(auditUtentiFirma);

                            break;
                        case Images.Checklist:
                            AuditChecklist auditChecklist = await App.Database.GetAuditChecklist(idAudit, chiaveTabella);
                            auditChecklist.Immagine = immagineReturn;
                            await App.Database.UpdateAuditChecklist(auditChecklist);

                            break;
                        case Images.Utenti:
                            AuditUtenti auditUtenti = await App.Database.GetAuditUtentiByAudit(idAudit);
                            auditUtenti.Immagine = immagineReturn;
                            await App.Database.UpdateAuditUtenti(auditUtenti);

                            break;
                        case Images.Persone:
                            AuditPersone auditPersone = await App.Database.GetAuditPersone(idAudit, chiaveTabella);
                            auditPersone.Immagine = immagineReturn;
                            await App.Database.UpdateAuditPersone(auditPersone);

                            break;
                        case Images.Icone:
                            break;
                        default:
                            break;
                    }
                }
                catch (Exception ex)
                {
                    await Log(nameof(DownloadImagesAudit), ex.Message);
                    retValue = false;
                }
            }

            return retValue;
        }

        public static (double X, double Y) GetScreenCoordinates(VisualElement view)
        {
            // A view's default X- and Y-coordinates are LOCAL with respect to the boundaries of its parent,
            // and NOT with respect to the screen. This method calculates the SCREEN coordinates of a view.
            // The coordinates returned refer to the top left corner of the view.

            // Initialize with the view's "local" coordinates with respect to its parent
            double screenCoordinateX = view.X;
            double screenCoordinateY = view.Y;

            // Get the view's parent (if it has one...)
            if (view.Parent.GetType() != typeof(App))
            {
                VisualElement parent = (VisualElement)view.Parent;


                // Loop through all parents
                while (parent != null)
                {
                    // Add in the coordinates of the parent with respect to ITS parent
                    screenCoordinateX += parent.X;
                    screenCoordinateY += parent.Y;

                    // If the parent of this parent isn't the app itself, get the parent's parent.
                    if (parent.Parent.GetType() == typeof(App))
                        parent = null;
                    else
                        parent = (VisualElement)parent.Parent;
                }
            }

            // Return the final coordinates...which are the global SCREEN coordinates of the view
            return (screenCoordinateX, screenCoordinateY);
        }

        public static string GetColoreRisposta(string risposta, int idTipoDomanda)
        {
            string retValue = string.Empty;

            if (string.IsNullOrEmpty(risposta))
                retValue = "FilterBackgroundColor";
            else
            {
                switch (idTipoDomanda)
                {
                    case 1:
                    case 2:
                    case 9:
                    case 10:
                    case 11:
                        switch (risposta)
                        {
                            case "1":
                                retValue = Constants.ColoreRispostaOK;
                                break;
                            case "2":
                                retValue = Constants.ColoreRispostaNA;
                                break;
                            case "3":
                                retValue = Constants.ColoreRispostaNT;
                                break;
                            case "0":
                                retValue = Constants.ColoreRispostaKO;
                                break;
                        }

                        break;
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                        retValue = Constants.ColoreRispostaOK;
                        break;
                }
            }

            return retValue;
        }

        public static async Task UpdateStatoAudit(Audit audit, int nuovoStato)
        {
            audit.IdStato = nuovoStato;
            audit.DataProgress = DateTime.Now;

            await App.Database.UpdateAudit(audit);
        }
    }
}