﻿using AuditManager.Exceptions;
using AuditManager.Helpers;
using AuditManager.Interfaces;
using AuditManager.Models;
using AuditManager.Models.Sync;
using AuditManager.Models.Tables;
using AuditManager.Shared.Constants;
using AuditManager.Shared.Helpers;
using AuditManager.Shared.Types;
using JWT.Algorithms;
using JWT.Builder;
using ModernHttpClient;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;
using System;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace AuditManager.Services
{
    public class NetworkService : INetworkService
    {
        private HttpClient client;
        private readonly JsonSerializerSettings _serializerSettings;

        private CancellationTokenSource cancellationTokenSource;
        public CancellationTokenSource CancellationTokenSource
        {
            get => cancellationTokenSource;
            set => cancellationTokenSource = value;
        }

        private TimeSpan Timeout;

        #region Ctors
        public NetworkService()
        {
            _serializerSettings = new JsonSerializerSettings
            {
                ContractResolver = new CamelCasePropertyNamesContractResolver(),
                DateTimeZoneHandling = DateTimeZoneHandling.Utc,
                NullValueHandling = NullValueHandling.Ignore
            };
            _serializerSettings.Converters.Add(new StringEnumConverter());
        }
        #endregion


        #region Common Methods
        private HttpClient CreateHttpClient(string token)
        {
            Timeout = new TimeSpan(0, 0, 100);
            var messageHandler = new NativeMessageHandler()
            {
                Timeout = Timeout,
                EnableUntrustedCertificates = true,
                DisableCaching = true
            };

            client = new HttpClient(messageHandler);
            CancellationTokenSource = new CancellationTokenSource();
            CancellationTokenSource.CancelAfter(Timeout);

            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            if (!string.IsNullOrEmpty(token))
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", token);

            return client;
        }

        private string GetUserTokenAsync(string user, string deviceId)
        {
            if (string.IsNullOrEmpty(user))
                user = string.Empty;

            if (string.IsNullOrEmpty(deviceId))
                deviceId = string.Empty;

            string token = new JwtBuilder()
                .WithAlgorithm(new HMACSHA256Algorithm())
                .WithSecret(Constants.Secret)
                .AddClaim("exp", DateTimeOffset.UtcNow.AddHours(1).ToUnixTimeSeconds())
                .AddClaim("user", user)
                .AddClaim("deviceId", deviceId)
                .Build();

            return token;
        }

        private async Task<string> HandleResponse(HttpResponseMessage response)
        {
            string content = await response.Content.ReadAsStringAsync();

            if (!response.IsSuccessStatusCode)
            {
                if (response.StatusCode == HttpStatusCode.Forbidden || response.StatusCode == HttpStatusCode.Unauthorized)
                    throw new ServiceAuthenticationException(content);

                throw new HttpRequestExceptionEx(response.StatusCode, content);
            }

            return content;
        }

        private ApiResponse<T> ToFailedResponse<T>(string message, ApiStatus status)
        {
            return new ApiResponse<T>() { ErrorMessage = message, Status = status };
        }
        #endregion


        #region Login And Logout
        public async Task<ApiResponse<Utenti>> Login(LoginModel lm)
        {
            if (!Settings.IsConnected)
                return ToFailedResponse<Utenti>(string.Empty, ApiStatus.NoConnectivity);

            lm.Password = Cipher.Encrypt(lm.Password, Constants.PasswordCipher);

            string json = JsonConvert.SerializeObject(lm);
            string token = GetUserTokenAsync(string.Empty, Settings.DeviceId);
            string loginUrl = string.Format(Constants.BaseUrl, Constants.AccountUrl, Constants.LoginMethod);

            lm.Password = Cipher.Decrypt(lm.Password, Constants.PasswordCipher);

            try
            {
                HttpClient httpClient = CreateHttpClient(token);
                HttpContent content = new StringContent(json, Encoding.UTF8, "application/json");

                HttpResponseMessage response = await httpClient.PostAsync(loginUrl, content, CancellationTokenSource.Token);

                string jsonString = await HandleResponse(response);

                return JsonConvert.DeserializeObject<ApiResponse<Utenti>>(jsonString.Trim());
            }
            catch (OperationCanceledException ex) when (!CancellationTokenSource.IsCancellationRequested)
            {
                return ToFailedResponse<Utenti>(ex.Message, ApiStatus.NoConnectivity);
            }
            catch (Exception ex)
            {
                return ToFailedResponse<Utenti>(ex.Message, ApiStatus.NoConnectivity);
            }
        }

        public async Task<ApiResponse<bool>> Logout()
        {
            if (!Settings.IsConnected)
                return ToFailedResponse<bool>(string.Empty, ApiStatus.NoConnectivity);

            string token = GetUserTokenAsync(Settings.User, Settings.DeviceId);
            string logoutUrl = string.Format(Constants.BaseUrl, Constants.AccountUrl, Constants.LogoutMethod);

            HttpClient httpClient = CreateHttpClient(token);

            try
            {
                HttpResponseMessage response = await httpClient.GetAsync(logoutUrl, CancellationTokenSource.Token);
                await HandleResponse(response);

                string jsonString = await response.Content.ReadAsStringAsync();
                ApiResponse<bool> result = JsonConvert.DeserializeObject<ApiResponse<bool>>(jsonString, _serializerSettings);

                return result;
            }
            catch (Exception ex)
            {
                return ToFailedResponse<bool>(ex.Message, ApiStatus.Exception);
            }
        }
        #endregion

        #region Initial Sync
        public async Task<ApiResponse<bool>> GetDatabaseFromServer(bool isLogin = true)
        {
            ApiResponse<bool> retValue = new ApiResponse<bool> { Response = true };

            if (Settings.IsConnected && Settings.IsOnline)
            {
                //TODO: POST DATABASE
                //TODO: RESYNC ??

                if (isLogin)
                {
                    Sync sync = new Sync();
                    sync.UpdateSyncValues(true);
                    retValue = await PostSyncAsync(sync);
                }
                else
                {
                    int contaTipiAudit = await App.Database.ContaTipiAudit();

                    if (contaTipiAudit == 0)
                    {
                        Sync sync = new Sync();
                        sync.UpdateSyncValues(true);
                        retValue = await PostSyncAsync(sync);
                    }
                }

                if (retValue.Response)
                    retValue = await GetCatenaTabelleAccessorieAsync();

                //TODO: GET AUDIT NON ASSOCIATI E RELATIVO POST
                //TODO: GET CATENA PROGRESS E RELATIVO POST

                if (retValue.Response)
                    retValue = await GetCatenaAuditAsync(isLogin);

                //TODO: POST AUDIT SCARICATI

                //TODO: GET AUDIT CANCELLATI E RELATIVO POST

                if (retValue.Response)
                    await App.Database.UpdateParametroAsync<DateTime>(Constants.ConstDtUltimoAggiornamento, DateTime.Now);
            }

            return retValue;
        }

        public async Task<ApiResponse<bool>> GetCatenaTabelleAccessorieAsync()
        {
            if (!Settings.IsConnected)
                return ToFailedResponse<bool>(string.Empty, ApiStatus.NoConnectivity);

            string token = GetUserTokenAsync(Settings.User, Settings.DeviceId);
            string urlFinale = string.Format(Constants.BaseUrl, Constants.SyncUrl, Constants.SyncTabelleAccessorieMethod);

            HttpClient httpClient = CreateHttpClient(token);
            HttpResponseMessage response = new HttpResponseMessage();
            Uri uri = new Uri(urlFinale);

            ApiResponse<bool> retValue = new ApiResponse<bool>();

            try
            {
                response = await httpClient.GetAsync(uri);
                await HandleResponse(response);
                string serialized = await response.Content.ReadAsStringAsync();
                ApiResponse<TabelleAccessorieJson> taj = await Task.Run(() => JsonConvert.DeserializeObject<ApiResponse<TabelleAccessorieJson>>(serialized));

                int result = await App.Database.InsertTabelleAccessorie(taj.Response);

                switch (result)
                {
                    case 0:
                        retValue.Response = true;
                        retValue.Status = ApiStatus.Ok;

                        break;
                    case -1:
                        retValue.Response = false;
                        retValue.Status = ApiStatus.Error;

                        break;
                    default:
                        taj.Response.Sync.UpdateSyncValues(false);

                        ApiResponse<bool> postValue = await PostSyncAsync(taj.Response.Sync);
                        retValue.Response = postValue.IsSuccess;
                        retValue.Status = (postValue.IsSuccess) ? ApiStatus.Ok : ApiStatus.Error;
                        retValue.ErrorMessage = postValue.ErrorMessage;

                        break;
                }

                return retValue;
            }
            catch (Exception ex)
            {
                return ToFailedResponse<bool>(ex.Message, ApiStatus.Exception);
            }
        }

        public async Task<ApiResponse<bool>> PostSyncAsync(Sync sync)
        {
            string urlFinalePost = string.Format(Constants.BaseUrl, Constants.SyncUrl, Constants.PostSyncTabelleAccessoorieMethod);

            ApiResponse<bool> postValue = await PostAsync(urlFinalePost, sync);
            return postValue;
        }

        public async Task<ApiResponse<bool>> GetCatenaAuditAsync(bool isLogin = false)
        {
            if (!Settings.IsConnected)
                return ToFailedResponse<bool>(string.Empty, ApiStatus.NoConnectivity);

            string token = GetUserTokenAsync(Settings.User, Settings.DeviceId);
            string urlFinale = string.Format(Constants.BaseUrl, Constants.SyncUrl, Constants.SyncCatenaAuditMethod);

            HttpClient httpClient = CreateHttpClient(token);
            HttpResponseMessage response = new HttpResponseMessage();
            Uri uri = new Uri(urlFinale);

            ApiResponse<bool> retValue = new ApiResponse<bool>();

            try
            {
                response = await httpClient.GetAsync(uri);
                await HandleResponse(response);
                string serialized = await response.Content.ReadAsStringAsync();
                ApiResponse<TabellaJson> taj = await Task.Run(() => JsonConvert.DeserializeObject<ApiResponse<TabellaJson>>(serialized));

                int result = await App.Database.InsertCatenaAudit(taj.Response, isLogin);

                switch (result)
                {
                    case 0:
                        retValue.Response = true;
                        retValue.Status = ApiStatus.Ok;

                        break;
                    case -1:
                        retValue.Response = false;
                        retValue.Status = ApiStatus.Error;

                        break;
                    default:
                        //taj.Response.Sync.UpdateSyncValues(false);

                        //ApiResponse<bool> postValue = await PostSyncAsync(taj.Response.Sync);
                        //retValue.Response = postValue.IsSuccess;
                        //retValue.Status = (postValue.IsSuccess) ? ApiStatus.Ok : ApiStatus.Error;
                        //retValue.ErrorMessage = postValue.ErrorMessage;

                        retValue.Response = true;
                        retValue.Status = ApiStatus.Ok;

                        break;
                }

                foreach (var item in taj.Response.Audit)
                {
                    await FunctionsHelper.DownloadImagesAudit(item.IdAudit);
                }

                return retValue;
            }
            catch (Exception ex)
            {
                return ToFailedResponse<bool>(ex.Message, ApiStatus.Exception);
            }
        }

        public Task RefreshDatabaseAsync()
        {
            throw new NotImplementedException();
        }
        #endregion

        #region Post Methods
        public async Task<ApiResponse<bool>> PostAsync<T>(string url, T data, string header = "")
        {
            if (!Settings.IsConnected)
                return ToFailedResponse<bool>(string.Empty, ApiStatus.NoConnectivity);

            string token = GetUserTokenAsync(Settings.User, Settings.DeviceId);
            HttpClient httpClient = CreateHttpClient(token);
            Uri uri = new Uri(url);

            try
            {
                var content = new StringContent(JsonConvert.SerializeObject(data));
                content.Headers.ContentType = new MediaTypeHeaderValue("application/json");

                HttpResponseMessage response = httpClient.PostAsync(uri, content, CancellationTokenSource.Token).GetAwaiter().GetResult();
                await HandleResponse(response);

                string serialized = await response.Content.ReadAsStringAsync();
                ApiResponse<bool> result = JsonConvert.DeserializeObject<ApiResponse<bool>>(serialized, _serializerSettings);

                return result;
            }
            catch (Exception ex)
            {
                return ToFailedResponse<bool>(ex.Message, ApiStatus.Exception);
            }
        }
        #endregion
    }
}
